package de.cnam.jemina.database;

import de.cnam.jemina.database.server.EmbeddedHsqlServer;
import de.cnam.jemina.extern.auth.IUser;
import de.cnam.jemina.objects.Artikel;
import de.cnam.jemina.objects.Besitzer;
import de.cnam.jemina.objects.Buch;
import de.cnam.jemina.objects.CD;
import de.cnam.jemina.objects.Filiale;
import de.cnam.jemina.objects.Person;
import de.cnam.jemina.objects.Reservierung;
import de.cnam.jemina.objects.Video;
import de.cnam.jemina.objects.Zeitschrift;
import de.cnam.jemina.objects.factory.ObjectFactory;
import de.cnam.jemina.persistents.IPersistensObject;
import de.cnam.jemina.utils.ExpirationDateCalculator;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Speichert alle Daten in die {@link EmbeddedHsqlServer embedded HSQL-DB}.
 * @author CNAM Team (Nadine, Mike, Jens)
 * @todo: Diese Klasse ist Quick&Dirty entstanden... dringend ueberarbeiten!!!
 */
public class HSqlPersistents extends DatabasePersistents {
    
    // Logging:
    private static final Logger log = LoggerFactory.getLogger(HSqlPersistents.class);
    
    // Verbindungsdaten
    private static final String DRIVER = "org.hsqldb.jdbcDriver";
    private static final String USERNAME = "sa";
    private static final String PASSWORD = "";
        
    // URL
    private String url;
    
    // Connection Pooling

    /**
     * 
     * @param port Portnummer
     * @param dbname Datenbankname
     */
    public HSqlPersistents(int port, String dbname) {
        super();
        
        // JDBC URL bauen:
        url = "jdbc:hsqldb:hsql://localhost:" + port + "/" + dbname;
        
        // Treiber laden:
        try {            
            Class.forName(DRIVER);
        } catch (ClassNotFoundException ex) {
            log.error("Konnte Treiber-Klasse {} nicht laden!", DRIVER);
        }
        
        // Connection Pool vorbereiten:
        setupConnectionPool(url, USERNAME, PASSWORD);
        
        // Tabellen anlegen:
        try {
            createTabels(getConnection());
        } catch (Exception ex) {
            log.error("Fehler beim anlegen der Tabellen", ex);
        }
    }
    
    @Override
    // @todo: Spaltennamen bei 'getColumnWidth' als Konstante auslagern bzw. bessere Loesung finden
    public void insert(IPersistensObject obj, IUser user) throws SQLException {
        if (obj == null) {
            log.warn("Es wurde kein Objekt zum speichern uebergeben");
            return;
        }
        
        // Verbindung aufbauen:
        Connection con = getConnection();
        con.setAutoCommit(false);
        PreparedStatement pre = null;
        
        try {
            // Je nach Objekt entsprechende Methode aufrufen
            int rowid = -1;
            if (obj instanceof Artikel) {
                String typ = null;
                if (obj instanceof Buch) {
                    typ = ARTIKEL_BUCH;
                }
                if (obj instanceof Zeitschrift) {
                    typ = ARTIKEL_ZEITSCHRIFT;
                }
                if (obj instanceof CD) {
                    typ = ARTIKEL_CD;
                }
                if (obj instanceof Video) {
                    typ = ARTIKEL_VIDEO;
                }
                
                // Wegen einer vertikalen Paritionierung der Datenbank, sind die
                // Artikel-Daten in einer eigenen Tabelle
                Artikel a = (Artikel) obj;
                pre = con.prepareStatement(sqlManager.getSql(ARTIKEL_INSERT));
                pre.setString(1, StringUtils.abbreviate(a.getTitel(), getColumnWidth(TABLE_ARTIKEL, "Titel")));
                pre.setString(2, StringUtils.abbreviate(a.getBeschreibung(), getColumnWidth(TABLE_ARTIKEL, "Beschreibung")));                
                pre.setString(3, typ);
                pre.setLong(4, System.currentTimeMillis());
                pre.setString(5, StringUtils.abbreviate(user.getLoginId(), getColumnWidth(TABLE_ARTIKEL, "ERSTELLT_VON")));
                pre.execute();
                
                // Jetzt RowId lesen
                rowid = getRowId(con, a);
                
                // Buch
                if (obj instanceof Buch) {
                    Buch b = (Buch) obj;
                    pre = con.prepareStatement(sqlManager.getSql(ARTIKEL_INSERT_BUCH));
                    pre.setInt(1, rowid);
                    pre.setString(2, StringUtils.abbreviate(b.getAutoren(), getColumnWidth(TABLE_BUCH, "AUTOREN")));
                    pre.setString(3, StringUtils.abbreviate(b.getIsbn(), getColumnWidth(TABLE_BUCH, "ISBN")));
                }

                // @todo: Die anderen Artikel-Arten ...
            }

            // Besitzer (Filiale)
            if (obj instanceof Filiale) {
                pre = con.prepareStatement(sqlManager.getSql(BESITZER_INSERT));
                pre.setString(1, BESITZER_FILIALE);
                pre.execute();
                
                // ID laden
                // @todo: Gefaehrlich im Multi-User-Betrieb!!!!!!!
                int id = -1;
                pre = con.prepareStatement(sqlManager.getSql(BESITZER_SELECT_MAXID));
                try (ResultSet rs = pre.executeQuery()) {
                    if (rs.next()) {
                        id = rs.getInt(1);
                    }
                }
                
                // Filiale einfuegen
                pre = con.prepareStatement(sqlManager.getSql(BESITZER_INSERT_FILIALE));
                pre.setInt(1, id);
                pre.setString(2, ((Filiale)obj).getName());
                pre.setInt(3, ((Filiale)obj).getPlz());
                pre.setString(4, ((Filiale)obj).getOrt());
                pre.setString(5, ((Filiale)obj).getStrasse());
            }
            
            
            // Commit:
            if (pre != null) {
                pre.execute();
            }
            con.commit();            
        } catch (Exception ex) {
            con.rollback();
            throw ex;
        } finally {        
            // Verbindung schliessen:
            con.close();
        }
    }

    @Override
    public void update(IPersistensObject obj, IUser user) throws SQLException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void delete(IPersistensObject obj, IUser user) throws SQLException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public List<Artikel> selectArticle(String searchtxt) throws SQLException {
        List<Artikel> list = new ArrayList<>();
        
        if (searchtxt == null) {
            searchtxt = StringUtils.EMPTY;
        }
        
        Connection con = getConnection();
        try {
            PreparedStatement pre = con.prepareStatement(sqlManager.getSql(ARTIKEL_SELECT_BYFILTER));
            pre.setString(1, "%"+searchtxt.toLowerCase()+"%");
            pre.setString(2, "%"+searchtxt.toLowerCase()+"%");
            try (ResultSet rs = pre.executeQuery()) {
                Artikel a;
                String typ, titel, beschreibung;
                int id;
                while (rs.next()) {
                    typ = rs.getString("typ");
                    titel = rs.getString("titel");
                    beschreibung = rs.getString("beschreibung");
                    id = rs.getInt("artikel_id");
                    a = ObjectFactory.genArtikel(id, titel, beschreibung, typ);
                    
                    if (a != null) {
                        list.add(a);
                    }
                }
            }
        } catch (SQLException ex) {
            throw ex;
        } finally {
            if (con != null) {
                con.close();
            }
        }
        
        return list;
    }

/*
    @Override
    public List<ArtikelListObject> selectArticleWithLocation(String searchtxt) throws SQLException {
        List<ArtikelListObject> list = new ArrayList<>();
        
        if (searchtxt == null) {
            searchtxt = StringUtils.EMPTY;
        }
        
        Connection con = getConnection();
        try {
            // Lade Besitzer
            Map<Integer, Besitzer> besitzer = getBesitzer();
            
            PreparedStatement pre = con.prepareStatement(sqlManager.getSql(ARTIKEL_SELECT_BYFILTER_WITH_OWNER));
            pre.setString(1, "%"+searchtxt.toLowerCase()+"%");
            pre.setString(2, "%"+searchtxt.toLowerCase()+"%");
            ResultSet rs = pre.executeQuery();
            Artikel a;
            ArtikelListObject aobj;
            String typ, titel, beschreibung;
            int id;
            while (rs.next()) {
                typ = rs.getString("typ");
                titel = rs.getString("titel");
                beschreibung = rs.getString("beschreibung");
                id = rs.getInt("artikel_id");
                a = ObjectFactory.genArtikel(id, titel, beschreibung, typ);
                
                if (a != null) {
                    aobj = new ArtikelListObject(a, 0, 0, null);
                    
                }
            }
            rs.close();
        } catch (SQLException ex) {
            throw ex;
        } finally {
            if (con != null) {
                con.close();
            }
        }
        
        return list;
    }
*/
    
    /**
     * Liefert eine {@link Map} mit den Besitzer_ID's als Key und dem
     * entsprechenden Objekt als Value.
     * @return Liefert kein null zur&uuml;ck!
     */
    protected Map<Integer, Besitzer> getBesitzer() throws SQLException {
        Map<Integer, Besitzer> res = new HashMap<>();
        
        Connection con = getConnection();
        try {
            PreparedStatement pre = con.prepareStatement(sqlManager.getSql(BESITZER_SELECT));
            PreparedStatement prePerson = con.prepareStatement(sqlManager.getSql(BESITZER_SELECT_PERSON_BYID));
            PreparedStatement preFiliale = con.prepareStatement(sqlManager.getSql(BESITZER_SELECT_FILIALE_BYID));
            try (ResultSet rs = pre.executeQuery()) {
                ResultSet rsSub;
                String typ;
                Integer id;
                Besitzer b;
                while (rs.next()) {
                    b = null;
                    id = rs.getInt("BESITZER_ID");
                    typ = rs.getString("TYP");
                    switch (typ) {
                        case BESITZER_FILIALE:
                            preFiliale.setInt(1, id);
                            rsSub = preFiliale.executeQuery();
                            if (rsSub.next()) {
                                b = new Filiale(id, rsSub.getString("NAME"), rsSub.getInt("PLZ"), rsSub.getString("ORT"), rsSub.getString("STRASSE"));
                            }
                            rsSub.close();
                            break;
                        case BESITZER_PERSON:
                            prePerson.setInt(1, id);
                            rsSub = prePerson.executeQuery();
                            if (rsSub.next()) {
                                b = new Person(id, rsSub.getString("USER_ID"));
                            }
                            rsSub.close();
                            break;
                        default:                        
                            log.warn("Unbekannter Besitzer-Typ: {}", typ);
                    }
                    
                    if (b != null) {
                        res.put(id, b);
                    }
                }
            }
        } catch (SQLException ex) {
            throw ex;
        } finally {
            if (con != null) {
                con.close();
            }
        }
        
        // Log
        if (log.isInfoEnabled()) {
            log.info("Es wurden {} Besitzer gefunden.", res.size());
        }
        
        return res;
    }

    @Override
    public Artikel loadArtikelDetails(int id) throws SQLException {
        Artikel a = null;
        Connection con = getConnection();
        try {
            PreparedStatement pre = con.prepareStatement(sqlManager.getSql(ARTIKEL_SELECT_BYID));
            pre.setInt(1, id);
            
            PreparedStatement subPre = null;            
            try (ResultSet rs = pre.executeQuery()) {
                ResultSet subRs;
                String typ, titel, beschreibung;
                if (rs.next()) {
                    typ = rs.getString("typ");
                    titel = rs.getString("titel");
                    beschreibung = rs.getString("beschreibung");
                    id = rs.getInt("artikel_id");
                    a = ObjectFactory.genArtikel(id, titel, beschreibung, typ);

                    if (a != null) {
                        // Nun Details laden
                        if (a instanceof Buch) {
                            subPre = con.prepareStatement(sqlManager.getSql(ARTIKEL_SELECT_BUCH_BYID));
                            subPre.setInt(1, id);
                            subRs = subPre.executeQuery();
                            if (subRs.next()) {
                                ((Buch)a).setAutoren(subRs.getString("autoren"));
                                ((Buch)a).setIsbn(subRs.getString("isbn"));
                            }
                            subRs.close();
                        }
                        
                        if (a instanceof CD) {
                            subPre = con.prepareStatement(sqlManager.getSql(ARTIKEL_SELECT_CD_BYID));
                            subPre.setInt(1, id);
                            subRs = subPre.executeQuery();
                            if (subRs.next()) {
                                ((CD)a).setAbspieldauer(subRs.getInt("abspieldauer"));
                            }
                            subRs.close();
                        }
                        
                        if (a instanceof Video) {
                            subPre = con.prepareStatement(sqlManager.getSql(ARTIKEL_SELECT_VIDEO_BYID));
                            subPre.setInt(1, id);
                            subRs = subPre.executeQuery();
                            if (subRs.next()) {
                                ((Video)a).setAbspieldauer(subRs.getInt("abspieldauer"));
                                ((Video)a).setFsk(subRs.getInt("fsk"));
                                switch (subRs.getString("medium")) {
                                    case VIDEO_TYPE_DVD:
                                        ((Video)a).setTyp(Video.VideoTyp.DVD);
                                        break;
                                    case VIDEO_TYPE_BLUERAY:
                                        ((Video)a).setTyp(Video.VideoTyp.BLUERAY);
                                        break;
                                    case VIDEO_TYPE_VHS:
                                        ((Video)a).setTyp(Video.VideoTyp.VHS);
                                        break;
                                }
                            }
                            subRs.close();                            
                        }
                        
                        if (a instanceof Zeitschrift) {
                            subPre = con.prepareStatement(sqlManager.getSql(ARTIKEL_SELECT_ZEITSCHRIFT_BYID));
                            subPre.setInt(1, id);
                            subRs = subPre.executeQuery();
                            if (subRs.next()) {
                                ((Zeitschrift)a).setHerausgeber(subRs.getString("herausgeber"));
                            }
                            subRs.close();
                        }
                    }
                }
            }
        } catch (SQLException ex) {
            throw ex;
        } finally {
            if (con != null) {
                con.close();
            }
        }
        
        return a;
    }
    
    @Override
    public String reserve(Artikel a, Filiale f, IUser u) throws SQLException {
        if (a == null || f == null || u == null) {
            return null;
        }
        
        Connection con = getConnection();
        PreparedStatement pre;
        ResultSet rs;
        String rnummer = null;
        
        // @todo: Ein sehr grosser 'synchronized' Block !!
        synchronized("reserve") {
            try {
                // Schritt 1: Sind Nicht-Bestands-Exemplare am gewuenschten Ort?
                int count=0;
                pre = con.prepareStatement(sqlManager.getSql(RESERVIERUNG_STEP1));
                pre.setInt(1, a.getId());
                pre.setInt(2, f.getBesitzerId());
                rs = pre.executeQuery();
                if (rs.next()) {
                    count = rs.getInt(1);
                }
                rs.close();
                
                // Log
                if (log.isDebugEnabled()) {
                    log.debug("Es sind {} Exemplare des Artikels {} in der Filiale {} vorhanden.", count, a, f);
                }
                
                if (count > 0) {
                    // Schritt 2: Es sind also Exemplare vorhanden, sind welche bereits reserviert?
                    pre = con.prepareStatement(sqlManager.getSql(RESERVIERUNG_STEP2));
                    pre.setInt(1, a.getId());
                    pre.setInt(2, f.getBesitzerId());
                    rs = pre.executeQuery();
                    if (rs.next()) {
                        count -= rs.getInt(1);
                    }
                    rs.close();
                    
                    // Log
                    if (log.isInfoEnabled()) {
                        log.info("Es sind {} Exemplare des Artikels {} in der Filiale {} verfuegbar.", count, a, f);
                    }
                    
                    if (count > 0) {
                        // Fuehre nun Reservierung durch
                        Person p = getPerson(u);
                        if (p != null) {
                            // @todo: Die Reservierungsnummer muss ueberprueft werden (ob bereit vorhanden)
                            // Erstelle eine Reservierungsnummer
                            rnummer = RandomStringUtils.randomAlphanumeric(8);
                            
                            pre = con.prepareStatement(sqlManager.getSql(RESERVIERUNG_INSERT));
                            pre.setString(1, rnummer);
                            pre.setLong(2, System.currentTimeMillis());
                            pre.setInt(3, p.getBesitzerId());
                            pre.setLong(4, ExpirationDateCalculator.getReserveExpirationDate());
                            pre.setInt(5, a.getId());
                            pre.setInt(6, f.getBesitzerId());
                            
                            pre.execute();
                            con.commit();                            
                        } else {
                            log.error("Es konnte keine Person zum User {} gefunden werden.", u);
                        }
                    }
                }
            } catch (Exception ex) {
                log.error("Fehler bei der Reservierung", ex);                
            } finally {
                if (con != null) {
                    con.close();
                }
            }
        }
        
        return rnummer;
    }

    @Override
    public Person getPerson(IUser u) throws SQLException {
        if (u == null) {
            return null;
        }
        
        // Lade Person
        Person p = loadPerson(u);
        
        if (p == null) {
            // Person muss angelegt werden
            Connection con = getConnection();
            PreparedStatement pre;
            con.setAutoCommit(false);
            try {
                // Besitzer einfuegen
                pre = con.prepareStatement(sqlManager.getSql(BESITZER_INSERT));
                pre.setString(1, BESITZER_PERSON);
                pre.execute();
                
                // Neue Besitzer ID ermitteln
                // @todo: Gefaehrlich im Multi-User-Betrieb!!!!!!!
                int id = -1;
                pre = con.prepareStatement(sqlManager.getSql(BESITZER_SELECT_MAXID));
                try (ResultSet rs = pre.executeQuery()) {
                    if (rs.next()) {
                        id = rs.getInt(1);
                    }
                }
                
                // Person einfuegen:
                if (id > -1L) {
                    pre = con.prepareStatement(sqlManager.getSql(BESITZER_INSERT_PERSON));
                    pre.setInt(1, id);
                    pre.setString(2, u.getLoginId());
                    pre.execute();
                } else {
                    throw new Exception("Unbekannte ID");
                }
                
                // Commit
                con.commit();
                
                // Person lesen:
                p = loadPerson(u);
                
            } catch (Exception ex) {
                log.error("Es gab einen Fehler beim einfuegen einer Person:",ex);
                log.error("Personen-Objekt: {}", u);
                if (con != null) {                
                    con.rollback();
                }
            } finally {
                if (con != null) {                
                    con.close();
                }            
            }
        }
        
        return p;
    }
    
    /**
     * L&auml;dt eine Person aus der Datenbank
     * @param u
     * @return <b>null</b> wenn die Person nicht vorhanden ist
     * @throws SQLException 
     */
    protected Person loadPerson(IUser u) throws SQLException {
        if (u == null) {
            return null;
        }
        
        Person p = null;
        Connection con = getConnection();
        PreparedStatement pre;
        ResultSet rs;
        try {
            pre = con.prepareStatement(sqlManager.getSql(BESITZER_SELECT_PERSON_BYUSERID));
            pre.setString(1, u.getLoginId());
            rs = pre.executeQuery();
            if (rs.next()) {
                p = new Person(rs.getInt("besitzer_id"), rs.getString("user_id"));
            }
            rs.close();
        } catch (Exception ex) {
            log.error("Es gab einen Fehler beim Laden einer Person",ex);
        } finally {
            if (con != null) {                
                con.close();
            }            
        }
        
        return p;
    }
    /**
     * L&auml;dt eine Person aus der Datenbank
     * @param id ID der Person
     * @return <b>null</b> wenn die Person nicht vorhanden ist
     * @throws SQLException 
     */
    protected Person loadPerson(int id) throws SQLException {        
        Person p = null;
        Connection con = getConnection();
        PreparedStatement pre;
        ResultSet rs;
        try {
            pre = con.prepareStatement(sqlManager.getSql(BESITZER_SELECT_PERSON_BYID));
            pre.setInt(1, id);
            rs = pre.executeQuery();
            if (rs.next()) {
                p = new Person(rs.getInt("besitzer_id"), rs.getString("user_id"));
            }
            rs.close();
        } catch (Exception ex) {
            log.error("Es gab einen Fehler beim Laden einer Person",ex);
        } finally {
            if (con != null) {                
                con.close();
            }            
        }
        
        return p;
    }
    
    
    /**
     * L&auml;dt eine Filiale aus der Datenbank
     * @param id ID der Filiale
     * @return <b>null</b> wenn die Filiale nicht vorhanden ist
     * @throws SQLException 
     */
    protected Filiale loadFiliale(int id) throws SQLException {        
        Filiale p = null;
        Connection con = getConnection();
        PreparedStatement pre;
        ResultSet rs;
        try {
            pre = con.prepareStatement(sqlManager.getSql(BESITZER_SELECT_FILIALE_BYID));
            pre.setInt(1, id);
            rs = pre.executeQuery();
            if (rs.next()) {
                p = new Filiale(id, rs.getString("name"), rs.getInt("plz"), rs.getString("ort"), rs.getString("strasse"));
            }
            rs.close();
        } catch (Exception ex) {
            log.error("Es gab einen Fehler beim Laden einer Filiale",ex);
        } finally {
            if (con != null) {                
                con.close();
            }            
        }
        
        return p;
    }

    @Override
    public void executeSql(String sql) throws SQLException {
        Connection con = getConnection();
        try {
            PreparedStatement pre = con.prepareStatement(sql);
            pre.execute();
            con.commit();
        } catch (SQLException ex) {
            log.error("Fehler", ex);
            throw ex;
        } finally {
            if (con != null) {
                con.close();
            }
        }
    }

    @Override
    public Reservierung getReserve(String number) throws SQLException {
        if (StringUtils.isEmpty(number)) {
            return null;
        }
        
        Reservierung reservierung = null;
        
        Connection con = getConnection();
        try {
            PreparedStatement pre = con.prepareStatement(sqlManager.getSql(RESERVIERUNG_SELECT_BYNUMBER));
            pre.setString(1, number.toLowerCase());
            try (ResultSet rs = pre.executeQuery()) {
                if (rs.next()) {
                    // @todo: DUMMY-IMPLEMENTIERUNG !!
                    reservierung = new Reservierung(
                            rs.getLong("r_id"),
                            rs.getString("nummer"),
                            loadPerson(rs.getInt("besitzer_id")),
                            rs.getLong("ablauf"),
                            loadArtikelDetails(rs.getInt("artikel_id")),
                            loadFiliale(rs.getInt("aus_filiale")));
                }
            }
        } catch (SQLException ex) {
            log.error("Fehler", ex);
            throw ex;
        } finally {
            if (con != null) {
                con.close();
            }
        }
        
        return reservierung;
    }

    @Override
    public void delReserve(String number) throws SQLException {
        if (number != null) {
            Connection con = getConnection();
            try {
                PreparedStatement pre = con.prepareStatement(sqlManager.getSql(RESERVIERUNG_DELETE_BYNUMBER));
                pre.setString(1, number.toLowerCase());
                pre.execute();
                con.commit();
            } catch (SQLException ex) {
                log.error("Fehler", ex);
                throw ex;
            } finally {
                if (con != null) {
                    con.close();
                }
            }            
        }
    }

    @Override
    public void loan(Reservierung r, int exemplarId) throws SQLException {
        if (r != null) {
            Connection con = getConnection();
            try {
                int neuerBesitzer = r.getBesitzer().getBesitzerId();
                
                // Reservierung loeschen
                PreparedStatement pre = con.prepareStatement(sqlManager.getSql(RESERVIERUNG_DELETE_BYNUMBER));
                pre.setString(1, r.getNummer().toLowerCase());
                pre.execute();
                
                // Ausleihe erstellen
                // erstellt_am, besitzer_id, ablauf, exemplar_id, status
                pre = con.prepareStatement(sqlManager.getSql(AUSLEIHE_INSERT));
                pre.setLong(1, System.currentTimeMillis());
                pre.setInt(2, neuerBesitzer);
                pre.setLong(3, ExpirationDateCalculator.getLoanExpirationDate());
                pre.setInt(4, exemplarId);
                pre.setString(5, "");
                pre.execute();
                
                // Ort aendern:
                pre = con.prepareStatement(sqlManager.getSql(EXEMPLAR_CHANGE_ORT));
                pre.setInt(1, neuerBesitzer);
                pre.setInt(2, exemplarId);
                pre.execute();
                
                con.commit();
            } catch (SQLException ex) {
                con.rollback();
                log.error("Fehler", ex);
                throw ex;
            } finally {
                if (con != null) {
                    con.close();
                }
            }
        }
    }

    @Override
    public List<Integer> getExemplarIds(Artikel artikel, boolean withBestand) throws SQLException {
        // @todo: Filiale wird nicht beachtet!
        
        List<Integer> list = new ArrayList<>();
        if (artikel != null) {
            Connection con = getConnection();
            try {
                PreparedStatement pre = con.prepareStatement( sqlManager.getSql(withBestand ? EXEMPLAR_SELECT_BYARTIKELID : EXEMPLAR_SELECT_BYARTIKELIDWITHOUTBESTAND));
                pre.setInt(1, artikel.getId());
                try (ResultSet rs = pre.executeQuery()) {
                    while (rs.next()) {
                        list.add(rs.getInt("exemplar_id"));
                    }
                }
            } catch (SQLException ex) {
//                ex.printStackTrace();
                log.error("Fehler", ex);
                throw ex;
            } finally {
                if (con != null) {
                    con.close();
                }
            }
        }
        
        return list;
    }

    @Override
    public List<Reservierung> getReserveFromUser(IUser user) throws SQLException {
        List<Reservierung> result = new ArrayList<>();
        Connection con = getConnection();
        try {
            // Schritt 1: Besitzer-ID laden
            Person p = getPerson(user);
            
            // Schritt 2: Alle Reservierungen laden
            if (p != null) {
                PreparedStatement pre = con.prepareStatement( sqlManager.getSql(RESERVIERUNG_SELECT_BYUSER) );
                pre.setInt(1, p.getBesitzerId());
                try (ResultSet rs = pre.executeQuery()) {
                    Reservierung r = null;
                    while (rs.next()) {
                        // @todo:   Jetzt muss eigentlich eine Factory aktiv werden,
                        //          damit die ganze Objekt-Erstellung nicht noch mal
                        //          gemacht wird, gehen wir hier sehr unperfomant
                        //          auf eine andere Methode, die dann quasi wieder
                        //          alle Daten aus der DB laedt ... deshalb TODO!
                       r = getReserve(rs.getString("nummer"));
                       if (r != null) {
                           result.add(r);
                       }
                    }
                }
            }
            
        } catch (SQLException ex) {
            log.error("Fehler", ex);
            throw ex;
        } finally {
            if (con != null) {
                con.close();
            }
        }
        
        return result;
    }

    @Override
    public Filiale getFilialeByName(String name) throws SQLException {
        Filiale p = null;
        Connection con = getConnection();
        PreparedStatement pre;
        ResultSet rs;
        try {
            pre = con.prepareStatement(sqlManager.getSql(BESITZER_SELECT_FILIALE_BYNAME));
            pre.setString(1, name);
            rs = pre.executeQuery();
            if (rs.next()) {
                p = new Filiale(rs.getInt("besitzer_id"), rs.getString("name"), rs.getInt("plz"), rs.getString("ort"), rs.getString("strasse"));
            }
            rs.close();
        } catch (Exception ex) {
            log.error("Es gab einen Fehler beim Laden einer Filiale",ex);
        } finally {
            if (con != null) {                
                con.close();
            }            
        }
        
        return p;
    }

}
