package de.cnam.jemina.database.server;

import de.cnam.jemina.config.ConfigManager;
import de.cnam.jemina.database.HSqlPersistents;
import de.cnam.jemina.persistents.IPersistent;
import java.io.File;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.hsqldb.Server;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Diese Klasse startet und stoppt die interne HSQL Datenbank.
 * Dazu muss in der <b>web.xml</b> folgender Code eingetragen werden:
 *
 * <pre>
 * <listener>
 * <listener-class>de.cnam.jemina.database.server.EmbeddedHsqlServer</listener-class>
 * </listener>
 * </pre>
 * @author CNAM Team (Nadine, Mike, Jens)
 */
public class EmbeddedHsqlServer {

    // Logging:
    private static final Logger log = LoggerFactory.getLogger(EmbeddedHsqlServer.class);

    // HSQL Server:
    private Server hsqlServer;

    // Konfigurationsdateien:
    public static final String DATABASENAME = "jemina.db";
    public static final int DEFAULT_PORT = 9042;
    
    // Namen der Konfiguration
    private String CONFIG_DBPATH = "db.config.path";
    private String CONFIG_DBPORT = "db.config.port";
    private String CONFIG_DBNAME = "db.config.dbname";
    
    // Einstellungen:
    private String path; 
    private String dbname;        
    private int dbport;
    
    // Verbindungsobjekt
    private IPersistent con;
    
    // Singleton
    private static EmbeddedHsqlServer server;


    /**
     * Konstruktor
     */
    private EmbeddedHsqlServer() {
        super();
        loadConfig();
        startDatabase();
    }
    
    /**
     * Liefert eine Instanz dieser Klasse
     * @return 
     */
    public static synchronized EmbeddedHsqlServer getInstance() {
        if (server == null) {
            server = new EmbeddedHsqlServer();
        }
        return server;
    }
    
    
    
    /**
     * Startet die Datenbank
     */
    private void startDatabase() {        
        if (log.isInfoEnabled()) {
            log.info("Datenbank wird gestartet ...");
            log.info("Pfad fuer die Datenbank: " + path);
            log.info("Port fuer die Datenbank: " + dbport);
            log.info("Dateiname der Datenbank: " + dbname);
        }

        try {
            // Pfad anlegen (falls nicht vorhanden)
            new File(path).mkdirs();
            
            // Erzeuge Server-Objekt:
            hsqlServer = new Server();

            // Setze Log-Modus
            hsqlServer.setSilent(true);

            // Datenbank konfigurieren:
            hsqlServer.setDatabaseName(0, dbname);
            hsqlServer.setDatabasePath(0, path);

            // Setze Port
            hsqlServer.setPort(dbport);

            // Starte Datenbank:
            hsqlServer.start();

            // Log
            if (log.isInfoEnabled()) {
                log.info("Datenbank gestartet!");
            }
            
            // Lege Persistierungs-Objekt an:
            con = new HSqlPersistents(dbport, dbname);

        } catch (Exception ex) {
            log.error("Fehler beim starten der Datenbank!" , ex);
        }
    }

    /**
     * L&auml;dt die Einstellungen
     */
    private void loadConfig() {
        // ConfigManager
        ConfigManager cm = ConfigManager.getInstance();
        
        // Pfad ermitteln:
        final String lvConfigPath = cm.getConfig(CONFIG_DBPATH);
        
        // Port
        dbport = NumberUtils.toInt(cm.getConfig(CONFIG_DBPORT), DEFAULT_PORT);
        
        // Pfad
        path = lvConfigPath;
        
        // Datenbankname
        if (StringUtils.isEmpty(cm.getConfig(CONFIG_DBNAME))) {
            dbname = DATABASENAME;
        } else {
            dbname = cm.getConfig(CONFIG_DBNAME);
        }
    }

    /**
     * Stoppt die Datenbank
     */
    public void stopDatabase() {
        try {
            if (hsqlServer != null) {
                hsqlServer.signalCloseAllServerConnections();
                hsqlServer.shutdown();
                hsqlServer.stop();
                if (log.isInfoEnabled()) {
                    log.info("Datenbank beendet.");
                }
            }
        } catch (Exception ex) {
            log.error("Konnte Datenbank nicht stoppen!", ex);
        }
    }

    /**
     * Liefert ein Verbindungs-Objekt
     * @return 
     */
    public IPersistent getConnection() {
        return con;
    }
    
}


