package de.cnam.jemina.entrypoint;

import com.vaadin.Application;
import com.vaadin.service.ApplicationContext;
import com.vaadin.terminal.gwt.server.WebApplicationContext;
import com.vaadin.terminal.gwt.server.WebBrowser;
import com.vaadin.ui.Component;
import com.vaadin.ui.Window;
import de.cnam.jemina.database.server.EmbeddedHsqlServer;
import de.cnam.jemina.extern.auth.IUser;
import de.cnam.jemina.extern.auth.IUserDirectory;
import de.cnam.jemina.i18n.LocalizationManager;
import de.cnam.jemina.persistents.IPersistent;
import de.cnam.jemina.stub.auth.UserDirectory;
import de.cnam.jemina.ui.JeminaWindow;
import de.cnam.jemina.ui.controlls.ControllFactory;
import de.cnam.jemina.ui.controlls.IControll;
import java.util.HashMap;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <b>Einstiegspunkt der Anwendung</b><br/>
 * Die Anwendung steckt nun die einzelnen {@link IControll Module} zusammen,
 * und sorgt f&uuml;r die Komminikation zwischen diesen.
 * @author CNAM Team (Nadine, Mike, Jens)
 */
public class JeminaApplication extends Application implements IJeminaApplication {
    
    private static final String DEFAULT_THEME = "default";
    
    // Fuer die Uebersetzung:
    private final LocalizationManager locale = LocalizationManager.getLocalizationManager(JeminaApplication.class);

    // Logging:
    private final Logger log = LoggerFactory.getLogger(JeminaApplication.class);
    
    // User Directory
    private IUserDirectory userDir;
    private IUser user;
    
    // UI
    private JeminaWindow win;
    
    // Module-Register
    private Map<String, IControll> modules;
    
    // Module:
    private IControll modStartpage;
    private IControll modLogin;
    private IControll modLogout;
    private IControll modSearchResults;
    private IControll modListMyItems;
    private IControll modLoan;
    
    // Datenbank
    private EmbeddedHsqlServer server;
    private IPersistent persistents;

    /**
     * 
     */
    public JeminaApplication() {
        super();
        modules = new HashMap<>();
    }
    

    @Override
    public void init() {
        // Datenbank starten:
        server = EmbeddedHsqlServer.getInstance();
        persistents = server.getConnection();
        
        // Setze Theme
        setTheme(DEFAULT_THEME);
        
        // Einige Informationen ueber den User auslesen und ausgeben:
        ApplicationContext context = getContext();
        if (log.isInfoEnabled() && context instanceof WebApplicationContext) {
            WebApplicationContext webContext = (WebApplicationContext) context;
            WebBrowser browser = webContext.getBrowser();
            log.info("Aufruf des Systems:");
            log.info("IP-Adress: {}", browser.getAddress());
            log.info("User-agent: {}", browser.getBrowserApplication());
            log.info("Locale: {}", browser.getLocale());
        }
        
        // Testklasse die das Fremdsystem simuliert:
        userDir = new UserDirectory();
        
        // Module erstellen:        
        modLogin = ControllFactory.createLoginControll(this, userDir);
        modLogout = ControllFactory.createLogoutControll(this);
        modSearchResults = ControllFactory.createSearchResultsControll(this);
        modStartpage = ControllFactory.createStartpageControll(this);
        modListMyItems = ControllFactory.createListMyItemsControll(this);
        modLoan = ControllFactory.createLoanControll(this);
        
        // Layout (Window) erstellen
        win = new JeminaWindow(this, modLogin, modLogout);
        setMainWindow(win);
        
        // Module registrieren:
        register(modStartpage);
        register(modLogin);
        register(modLogout);
        register(modSearchResults);
        register(modListMyItems);
        register(modLoan);
        
        // Module der Navigation hinzufuegen:
        win.addModul(modStartpage);
        
        // Startseite aufrufen:        
        win.setPageContent(modStartpage.open(null, null));
    }

    @Override
    public void openModul(String name, Object obj, Map additionalParameters) {
        IControll controll = modules.get(name);
        if (controll == null) {
            log.error("Unbekanntes Modul: '{}'", name);
            return;
        }
        
        Component comp = controll.open(obj, additionalParameters);
        if (comp != null) {
            win.setPageContent(comp);
        }
    }

    @Override
    public IUser getUser() {
        return user;
    }

    @Override
    public IPersistent getPersistentObj() {
        return persistents;
    }

    @Override
    public void showNotification(String txt, int type) {
        win.showNotification(txt, type);
    }

    @Override
    public void showError(String txt) {
        showNotification(txt, Window.Notification.TYPE_ERROR_MESSAGE);
    }
    
    @Override
    public void showWarning(String txt) {
        showNotification(txt, Window.Notification.TYPE_WARNING_MESSAGE);
    }

    @Override
    public void showInfo(String txt) {
        showNotification(txt, Window.Notification.TYPE_HUMANIZED_MESSAGE);
    }

    @Override
    public void notifyLogin(IUser user) {
        this.user = user;
        
        // Module entfernen:
        win.removeAllModuls();
        
        // Module hinzufuegen:
        win.addModul(modStartpage);
        
        // User-Abhaengige Einstellungen:
        if (user != null) {
            // Dies wuerde man natuerlich nie so machen, da der Themename aus dem
            // Fremdsystem kommt. Zumindest eine Ueberpruefung muesste hier jetzt
            // hin.
            setTheme(user.getTheme());
            
            // Module
            win.addModul(modListMyItems);
            
            if (user.getRole() == IUser.ROLLE.MITARBEITER) {
                win.addModul(modLoan);
            }           
            
        } else {
            // Abgemeldet
            setTheme(DEFAULT_THEME);
        }
        
        // Zeige Startseite
        openModul(modStartpage.getName(), null, null);
    }
    
    /**
     * Registriert das &uuml;bergeben Modul
     * @param ct 
     */
    private void register(IControll ct) {
        modules.put(ct.getName(), ct);
    }

    @Override
    public Window getVaadinWindow() {
        return getMainWindow();
    }
    
}
